@@ -66,7 +66,7 @@ None of them fully met my needs.  In particular, this package has transparent
 support for:
 
     * chunked response encoding
-    * "async upload" of large request bodies
+    * streaming reads of large "async upload" requests
     * pluggable IO backends (e.g. eventlet, gevent)
 
 It's also designed from the ground up specifically for Mongrel2.  This means
@@ -88,6 +88,9 @@ Current bugs, limitations and things to do
 
 It's not all perfect just yet, although it does seem to mostly work:
 
+    * Needs tests something fierce!  I just have to find the patiences to
+      write all the setup and teardown cruft.
+
     * When running multiple threads, ctrl-C doesn't cleanly exit the process.
       Seems like the background threads get stuck in a blocking recv().
       I *really* don't want to emulate interrupts using zmq_poll...
@@ -96,7 +99,7 @@ It's not all perfect just yet, although it does seem to mostly work:
       ideal.  For example, it can schedule several fast requests to the same
       thread as a slow one, making them wait even if other threads become
       available.  I'm working on a zmq adapter that can do something better
-      (see the push2queue script in this distribution).
+      (see the pull2queue script in this distribution).
 
     * It would be great to grab connection details straight from the
       mongrel2 config database.  Perhaps a Connection.from_config method
@@ -106,8 +109,8 @@ It's not all perfect just yet, although it does seem to mostly work:
 """
 
 __ver_major__ = 0
-__ver_minor__ = 1
-__ver_patch__ = 1
+__ver_minor__ = 2
+__ver_patch__ = 0
 __ver_sub__ = ""
 __version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,__ver_patch__,__ver_sub__)
 
