@@ -12,15 +12,15 @@ classes from in here.
 import os
 import sys
 import json
+import thread
 import time
 import traceback
-import threading
 from collections import deque
 from cStringIO import StringIO
 
 import zmq
 
-from m2wsgi.util import pop_netstring, unquote_path, unquote
+from m2wsgi.util import pop_netstring, unquote_path, unquote, InputStream
 
 
 def monkey_patch():
@@ -90,18 +90,18 @@ class Connection(object):
     are specified, the sockets will run without identity in non-durable mode.
 
     If specified, the 'send_type' and 'recv_type' arguments give the type
-    of socket to use.  The defaults are zmq.PUSH and zmq.PUB respectively.
+    of socket to use.  The defaults are zmq.PULL and zmq.PUB respectively.
     Currently supported send socket types are:
 
-        * zmq.PUSH:  The standard push socket offered by Mongrel2.  This
+        * zmq.PULL:  The standard push socket offered by Mongrel2.  This
                      is very efficient but offers no way to detect handlers
                      that have disconnected.
 
         * zmq.REQ:   A request-response socket.  The handler explicitly asks
                      for new requests.  This is a little slower but allows
                      clients to cleanly disconnect (they just stop asking for
-                     requests).  For now, you must run the "push2queue" script
-                     included in this module to translate mongrel2's PUSH
+                     requests).  For now, you must run the "pull2queue" script
+                     included in this module to translate mongrel2's PULL
                      socket into an XREP socket to connect in this mode.
 
     Currently supported recv socket types are:
@@ -128,7 +128,7 @@ class Connection(object):
                 raise ValueError(msg)
             recv_spec = send_host + ":" + str(send_port - 1)
         if send_type is None:
-            send_type = zmq.PUSH
+            send_type = zmq.PULL
         elif isinstance(send_type,basestring):
             send_type = getattr(zmq,send_type)
         if recv_type is None:
@@ -141,11 +141,11 @@ class Connection(object):
         self.send_type = send_type
         self.recv_type = recv_type
         self.recv_buffer = deque()
-        self.reqs = self.ZMQ_CTX.socket(send_type)
+        self.reqs = self.ZMQ_CTX.socket(self.send_type)
         if self.send_ident is not None:
             self.reqs.setsockopt(zmq.IDENTITY,self.send_ident)
         self.reqs.connect(self.send_spec)
-        self.resps = self.ZMQ_CTX.socket(recv_type)
+        self.resps = self.ZMQ_CTX.socket(self.recv_type)
         if self.recv_ident is not None:
             self.resps.setsockopt(zmq.IDENTITY,self.recv_ident)
         self.resps.connect(self.recv_spec)
@@ -160,8 +160,8 @@ class Connection(object):
         (the default) or non-blocking recv.  If not blocking and no request
         is available, None is returned.
         """
-        #  For a PUSH socket, we just recv the message straight out of it.
-        if self.send_type == zmq.PUSH:
+        #  For a PULL socket, we just recv the message straight out of it.
+        if self.send_type == zmq.PULL:
             if blocking:
                 msg = self.reqs.recv()
             else:
@@ -322,6 +322,59 @@ class WSGIResponder(object):
 
 
 
+class StreamingUploadFile(InputStream):
+    """File-like object for streaming reads from in-progress uploads."""
+
+    def __init__(self,request,fileobj):
+        self.request = request
+        self.fileobj = fileobj
+        cl = request.headers.get("content-length","")
+        if not cl:
+            raise ValueError("missing content-length for streaming upload")
+        try:
+            cl = int(cl)
+        except ValueError:
+            msg = "malformed content-length for streaming upload: %r"
+            raise ValueError(msg % (cl,))
+        self.content_length = cl
+        super(StreamingUploadFile,self).__init__()
+
+    def close(self):
+        super(StreamingUploadFile,self).close()
+        self.fileobj.close()
+
+    def _read(self,sizehint=-1):
+        if self.fileobj.tell() >= self.content_length:
+            return None
+        if sizehint == 0:
+            data = ""
+        else:
+            if sizehint > 0:
+                data = self.fileobj.read(sizehint)
+                while not data:
+                    self._wait_for_data()
+                    data = self.fileobj.read(sizehint)
+            else:
+                data = self.fileobj.read()
+                while not data:
+                    self._wait_for_data()
+                    data = self.fileobj.read()
+        return data
+
+    def _wait_for_data(self):
+        """Wait for more data to be available from this upload.
+
+        The base implementation simply does a sleep loop until the
+        file grows past its current position.  Eventually we could try
+        using file notifications to detect change.
+        """
+        curpos = self.fileobj.tell()
+        cursize = os.fstat(self.fileobj.fileno()).st_size
+        while curpos >= cursize:
+            time.sleep(0.01)
+            cursize = os.fstat(self.fileobj.fileno()).st_size
+
+
 class WSGIHandler(object):
     """Mongrel2 Handler translating to WSGI.
 
@@ -345,6 +398,7 @@ class WSGIHandler(object):
 
     ConnectionClass = Connection
     ResponderClass = WSGIResponder
+    StreamingUploadClass = StreamingUploadFile
 
     def __init__(self,application,connection):
         self.started = False
@@ -369,20 +423,29 @@ class WSGIHandler(object):
                 self.serve_one_request()
         except Exception:
             self.running = False
-            raise
-        finally:
+            e1,e2,e3 = sys.exc_info()
+            try:
+                self._shutdown()
+            except Exception:
+                print >>sys.stderr, "------- shutdown error -------"
+                traceback.print_exc()
+                print >>sys.stderr, "------------------------------"
+            raise e1,e2,e3
+        else:
             self._shutdown()
 
     def _shutdown(self):
-        #  We have to handle anything in our recv queue,
-        #  or they will get lost when we close the socket.
+        #  We have to handle anything that's already in our recv queue,
+        #  or the requests will get lost when we close the socket.
         reqs = deque()
         req = self.connection.recv(blocking=False)
         while req is not None:
             reqs.append(req)
         self.connection.close_send()
         for req in reqs:
-            self.serve_one_request(req)
+            self.handle_request(req)
+        # TODO: handle_request might spawn a new thread, need to wait for
+        # them to compelete.
         self.connection.close()
 
     def stop(self):
@@ -391,14 +454,9 @@ class WSGIHandler(object):
         self.started = False
         # TODO: how to interrupt blocking recv?
     
-    def serve_one_request(self,req=None):
+    def serve_one_request(self):
         """Receive and serve a single request from Mongrel2."""
-        if req is None:
-            req = self.connection.recv()
-        #  Mongrel2 uses JSON requests internally.
-        #  We don't want them in our WSGI.
-        if req.headers.get("METHOD","") == "JSON":
-            return
+        req = self.connection.recv()
         self.handle_request(req)
         
     def handle_request(self,req):
@@ -408,22 +466,38 @@ class WSGIHandler(object):
         the mongrel2 request into a WSGI environ, invokes the application and
         sends the resulting response back to Mongrel2.
 
-        Subclasses way override this method to control how and when the request
+        Subclasses may override this method to control how and when the request
         is handled, e.g. by spawning a new thread or adding it to a work queue.
         It's a good idea to return control to the calling code as quickly as
         or you'll get a nice backlog of outstanding requests.
         """
-        #  If there's an async upload in progress, wait until
-        #  it completes.  Eventually we'll read this on demand.
-        if "x-mongrel2-upload-start" in req.headers:
-            if "x-mongrel2-upload-done" not in req.headers:
-                return
+        #  Mongrel2 uses JSON requests internally.
+        #  We don't want them in our WSGI.
+        if req.headers.get("METHOD","") == "JSON":
+            return
         #  OK, it's a legitimate full HTTP request.
         #  Route it through the WSGI app.
         environ = {}
         responder = self.ResponderClass(req)
         try:
-            #  Grab the real environ.
+            #  If there's an async upload in progress, we have two options.
+            #  If they sent a Content-Length header then we can do a streaming
+            #  read from the file as it is being uploaded.  If there's no
+            #  Content-Length then we have to wait for it all to upload (as
+            #  there's no guarantee that the same handler will get both the
+            #  start and end events for any upload).
+            if "x-mongrel2-upload-start" in req.headers:
+                if req.headers.get("content-length",""):
+                    #  We'll streaming read it on the -start event,
+                    #  so ignore the -done event.
+                    if "x-mongrel2-upload-done" in req.headers:
+                        return
+                else:
+                    #  We have to wait for the -done event,
+                    #  so ignore the -start event.
+                    if "x-mongrel2-upload-done" not in req.headers:
+                        return
+            #  Grab the full WSGI environ.
             #  This might error out, e.g. if someone tries any funny business
             #  with the mongrel2 upload headers.
             environ = self.get_wsgi_environ(req,environ)
@@ -439,8 +513,10 @@ class WSGIHandler(object):
                 if hasattr(chunks,"close"):
                     chunks.close()
         except Exception:
+            print >>sys.stderr, "------- request handling error -------"
             traceback.print_exc()
             sys.stderr.write(str(environ) + "\n\n")
+            print >>sys.stderr, "------------------------------ -------"
             #  Send an error response if we can.
             #  Always close the connection on error.
             if not responder.has_started:
@@ -471,15 +547,18 @@ class WSGIHandler(object):
             environ = {}
         #  Include keys required by the spec
         environ["REQUEST_METHOD"] = req.headers["METHOD"]
-        environ["SCRIPT_NAME"] = req.headers["PATTERN"]
-        while environ["SCRIPT_NAME"].endswith("/"):
-            environ["SCRIPT_NAME"] = environ["SCRIPT_NAME"][:-1]
-        environ["PATH_INFO"] = unquote_path(req.headers["PATH"])
+        script_name = req.headers.get("PREFIX",req.headers["PATTERN"])
+        while script_name.endswith("/"):
+            script_name = script_name[:-1]
+        environ["SCRIPT_NAME"] = unquote_path(script_name)
+        path_info = req.headers["PATH"][len(script_name):]
+        environ["PATH_INFO"] = unquote_path(path_info)
         if "QUERY" in req.headers:
             environ["QUERY_STRING"] = unquote(req.headers["QUERY"])
         environ["SERVER_PROTOCOL"] = req.headers["VERSION"]
         #  TODO: mongrel2 doesn't seem to send me this info.
         #  How can I obtain it?  Suck it out of the config?
+        #  Let's just hope the client sends a Host header...
         environ["SERVER_NAME"] = "localhost"
         environ["SERVER_PORT"] = "80"
         #  Include standard wsgi keys
@@ -520,18 +599,21 @@ class WSGIHandler(object):
         where an async upload is performed, it is the actual tempfile into
         which the upload was dumped.
 
-        Eventually, I plan to support reading from the async upload tempfile
-        while it is still being written, so that we get "streaming" of uploads
-        directly into the WSGI app.  Obviously, that's not trivial.
+        If the request contains a content-length, then we can read the upload
+        file while it is still comming in.  If not, we wait for it to
+        compelte and then use the raw file object.
         """
         upload_file = req.headers.get("x-mongrel2-upload-start",None)
         if not upload_file:
             return StringIO(req.body)
         upload_file2 = req.headers.get("x-mongrel2-upload-done",None)
-        if upload_file != upload_file2:
-            #  Highly suspicious behaviour; terminate immediately.
-            raise RuntimeError("mismatched mongrel2-upload header")
-        return open(upload_file,"rb")
+        if upload_file2 is None:
+            return self.StreamingUploadClass(req,open(upload_file,"rb"))
+        else:
+            if upload_file != upload_file2:
+                #  Highly suspicious behaviour; terminate immediately.
+                raise RuntimeError("mismatched mongrel2-upload header")
+            return open(upload_file,"rb")
 
 
 if __name__ == "__main__":
